#!/usr/bin/env python3

# -*- coding: utf-8 -*-

"""
Description: Create the exon_ctrl file and the exon file
"""

from pathlib import Path
import pandas as pd
from typing import List
from .config import Config, logging_def
import logging


def get_ctrl_exon(mfile: Path) -> List[str]:
    """
    Get the control exons in mfile.

    :param mfile: A file containing control exons
    :return: The lis of control exons in mfile
    """
    with mfile.open("r") as f:
        exon_list = f.read().splitlines()
    return exon_list


def get_ctrl(bed_file: Path) -> None:
    """
    Create a file containing the control exons

    :param bed_file: A bed file containing intern exons
    :return: the bed file as a dataframe
    """

    df = pd.read_csv(bed_file, sep="\t")
    df.columns = ["chr", "start", "stop", "name", "score", "strand"]
    df = df.loc[(df.stop - df.start > 2), :]
    df = df.loc[:, "name"]
    df.to_csv(Config.ctrl_exon_file, header=False, index=False)


def get_exon_table(ctrl_exon: Path, gene_tab: Path,
                   logging_level: str = "DISABLE"):
    """
    Create the table of exon.

    :param ctrl_exon: A file containing control exons
    :param gene_tab:  A file containing FasterDB gene display \
    in two columns: gene_symbol and fasterdb id.
    :param logging_level: The level of display
    """
    logging_def(Config.output, logging_level)
    ctrl_exon_list = get_ctrl_exon(ctrl_exon)
    logging.debug("Creating dataframe of control exons")
    df_exon = pd.DataFrame({"id": ctrl_exon_list})
    logging.debug(f"{df_exon.head()}\nshape:{df_exon.shape}")
    logging.debug("Creation of a pos column")
    df_exon["pos"] = df_exon["id"].apply(lambda x: int(x.split("_")[1]))
    logging.debug(f"{df_exon.head()}\nshape:{df_exon.shape}")
    logging.debug("Creation of a gene_id column")
    df_exon["gene_id"] = df_exon["id"].apply(lambda x: int(x.split("_")[0]))
    logging.debug(f"{df_exon.head()}\nshape:{df_exon.shape}")
    logging.debug("Addition of a gene column")
    gene_df = pd.read_csv(gene_tab, sep="\t")
    gene_df.columns = ["gene", "gene_id"]
    df_exon = df_exon.merge(gene_df)
    logging.debug(f"{df_exon.head()}\nshape:{df_exon.shape}")
    logging.debug("Addition of a name column")
    df_exon["name"] = df_exon.apply(lambda x: f"{x.gene}_"
                                              f"{x.pos}", axis=1)
    df_exon = df_exon[["pos", "name", "id", "gene_id"]]
    logging.debug(f"{df_exon.head()}\nshape:{df_exon.shape}")
    df_exon.to_csv(Config.exon_file, sep="\t", index=False)
